<?php

namespace Ulmus;

use Generator;

class EntityCollection extends \ArrayObject {
    
    public function filters(Callable $callback) : Generator
    {
        $idx = 0;
        
        foreach($this as $key => $item) {
            if ( $callback($item, $key, $idx) ) {
                $idx++;
                
                yield $item;
            }
        }
    }
        
    public function filtersCollection(Callable $callback) : self
    {
        $collection = new static();
        
        foreach($this->filters($callback) as $item) {
            $collection->append($item);
        }
        
        return $collection;
    }
    
    public function searchOne($value, string $field) : ? object 
    {
        # Returning first value only
        foreach($this->search($value, $field) as $item) {
            return $item;
        }
        
        return null;
    }
    
    public function search($value, string $field) : Generator 
    {
        foreach($this->filters(fn($v) => $v->$field === $value) as $item) {
            yield $item;
        }
    }
    
    public function buildArray(string $keyColumn, /* string|callable|null */ $value = null) : array
    {
        $list = [];
        
        foreach($this as $item) {
            switch (true) {
                case is_null($value):
                    $list[] = $item->$keyColumn;
                break;
                
                case is_callable($value):
                    $list[$item->$keyColumn] = $value($item);
                break;
                
                case is_object($value):
                case is_string($value):
                    $value = (string) $value;
                    
                    $list[$item->$keyColumn] = $item->$value;
                break;
            }
        }
        
        return $list;
    }
    
    public function toArray(bool $includeRelations = false) : array {
        $list = [];
        
        foreach($this as $entity) {
            $list[] = $entity->toArray($includeRelations);
        }
        
        return $list;
    }
}
